#ifndef __PV650X_H
#define __PV650X_H


#ifdef __DLL
#define _DECLSPEC __declspec(dllexport)
#endif
#ifndef __DLL
#define _DECLSPEC __declspec(dllimport)
#endif

#include <windows.h>
#include "FTD2XX.h"
#include "Fpga_Reg.h"

//------------------------------------------------------------------------------
#define MAX_BUF_LENGTH  20000   //     
#define CHANNEL_NUMBER  4       //   
#define MAX_PV_NUM      32      //    
#define MAX_SERIAL_LEN  32      //     
typedef char PV_SERIAL_LIST[ MAX_PV_NUM ][ MAX_SERIAL_LEN ];
//------------------------------------------------------------------------------
/*
================================================================================
                   
================================================================================
*/
/*
================================================================================
                    
================================================================================
*/
typedef union __TRIG_MODE {
unsigned short data  :10;
struct
{                         //----------------------------------------------------
  bool auto_restart :1;   //  (.   .)
  bool neg_sync     :1;   //   
  bool auto_sync    :1;   //  
  bool sync_pt      :1;   // C   
  bool sync_pt_in   :1;   // true -   
                          //    , flase - .
  bool sync_st      :1;   // C   
  bool sync_st_less :1;   //
  char trig_src     :3;   //     :
                          //----------------------------------------------------
#define   TRIG_A1      0  //    A1
#define   TRIG_A2      1  //    A2
#define   TRIG_D1      2  //    D1
#define   TRIG_D2      3  //    D2
#define   TRIG_DSB     4  //  

};} TRIG_MODE,*PTRIG_MODE;//
                          //----------------------------------------------------
typedef union __TRIG_COND{//   :
unsigned char data  :8;   //
struct                    //
{                         //----------------------------------------------------
 bool A1_lev  :1;         // 0 -  , 1 -  
 bool A2_lev  :1;         // 0 -  , 1 -  
 bool D1_lev  :1;         // 0 -  , 1 -  
 bool D2_lev  :1;         // 0 -  , 1 -  
                          //----------------------------------------------------
 bool A1_enb  :1;         //  A1
 bool A2_enb  :1;         //  A2
 bool D1_enb  :1;         //  D1
 bool D2_enb  :1;         //  D2
                          //----------------------------------------------------
};}TRIG_COND, *PTRIG_COND ;
/*
--------------------------------------------------------------------------------
                     
--------------------------------------------------------------------------------
*/
typedef union __ADC_MODE{//   :
unsigned char data  :7;   //
struct                    //
{                         //----------------------------------------------------
  bool peak_detect  :1;   //     
  bool select_A1    :1;   //   A1
  bool close_in_A1  :1;   //    1
  bool select_A2    :1;   //   A2
  bool close_in_A2  :1;   //    2
  bool select_D1    :1;   //   D1
  bool select_D2    :1;   //   D2
};} ADC_MODE , *PADC_MODE;//----------------------------------------------------

/*
--------------------------------------------------------------------------------
                         
--------------------------------------------------------------------------------
*/
typedef struct _OSC
{                         //----------------------------------------------------
  ADC_MODE adc_mode;      //     
                          //----------------------------------------------------
  TRIG_MODE trig_mode;    //     
                          //----------------------------------------------------
  TRIG_COND trig_cond;    //      
                          //----------------------------------------------------
  ULONG   smpl_per;       //    10ns
                          //----------------------------------------------------
  USHORT    smpl_pre;       //  
  USHORT    smpl_post;      //    
                          //----------------------------------------------------
  UCHAR   att_A1;         //  :         A1
  UCHAR   att_A2;         //  :         A2
                          //----------------------------------------------------
                          //   
                          //----------------------------------------------------
#define ATT_2VDiv       5 //
#define ATT_1VDiv       4 //
#define ATT_05VDiv      3 //
#define ATT_02VDiv      2 //
#define ATT_01VDiv      1 //
#define ATT_005VDiv     0 //
                          //----------------------------------------------------
  USHORT    zero_A1;        //   :       A1
  USHORT    zero_A2;        //   :       A2
                          //----------------------------------------------------
  UCHAR   trig_lev_A1;    //   :   A1
  UCHAR   trig_lev_A2;    //   :   A2
                          //----------------------------------------------------
  UCHAR   trig_gist_A1;   //   : A1
  UCHAR   trig_gist_A2;   //   : A2
                          //----------------------------------------------------
  UCHAR   trig_numb;      //      
                          //     ( )
                          //----------------------------------------------------
  UCHAR   trig_slew;      //  
                          //----------------------------------------------------
  ULONG   min_P;          //    
  ULONG   max_P;          //    ( ??? )
                          //----------------------------------------------------
  UCHAR   fmetr_in;       //   
                          //----------------------------------------------------
  UCHAR   fps;            //    ()
                          //----------------------------------------------------
/*
--------------------------------------------------------------------------------
                           
--------------------------------------------------------------------------------
*/                        //----------------------------------------------------
                          //   :
  bool    eq_flag;        //  -    
  bool    end_flag;       //  -     
  USHORT  addr_eq;        //  -     
                          //      
                          //----------------------------------------------------
  ULONG   trig_fine_pos;  //   
                          //     ( ??? )
                          //----------------------------------------------------
                          //   :
  UCHAR   before_V;       //  -     
  UCHAR   after_V;        //  -   
                          //     
  USHORT  trig_P;         //  -   
                          //     
  UCHAR   trig_S;         //  -  
  unsigned                //
  __int64 trig_gen_phase; //  -      .
                          //----------------------------------------------------
  double  freq;           //  
  bool    f_valid;        //  -   
                          //----------------------------------------------------
  bool    f_double_mem;   //  -   
                          //----------------------------------------------------
/*
--------------------------------------------------------------------------------
                               
--------------------------------------------------------------------------------
*/
                          //----------------------------------------------------
  bool    gen_enable;     //  
                          //----------------------------------------------------
  UCHAR   gen_mode;       //  
                          //----------------------------------------------------
#define GEN_MODE_CW          0  // 
#define GEN_MODE_SWIP        1  // 
#define GEN_MODE_AM          2  //  AM
#define GEN_MODE_FM          3  //  
#define GEN_MODE_SUM         4  // 
#define GEN_MODE_PULSE       5  // 
                          //----------------------------------------------------
  UCHAR   gen_mod_in;     //   
                          //----------------------------------------------------
#define GEN_MOD_INTERN    0x00  // 
#define GEN_MOD_CH_A1     0x10  //  1
#define GEN_MOD_CH_A2     0x20  //  2
#define GEN_MOD_CH_D1     0x30  //  D1
#define GEN_MOD_CH_D2     0x40  //  D2
                          //----------------------------------------------------
  bool    gen_mod_invert; //   
                          //----------------------------------------------------
  UCHAR   gen_shape;      //    
  UCHAR   gen_mod_shape;  //    
                          //----------------------------------------------------
#define GEN_WF_SIN           0  // 
#define GEN_WF_UP_TRIAN      1  //  //
#define GEN_WF_DN_TRIAN      2  //  \\
#define GEN_WF_RECT          3  // 
#define GEN_WF_TRIAN         4  // 
#define GEN_WF_DC            5  //  
                          //----------------------------------------------------
  USHORT  gen_phase;      //      
  USHORT  gen_mod_phase;  //    
                          // ( ).
                          //----------------------------------------------------
  double  gen_freq;       //  ,   
  double  gen_mod_freq;   // c     3.55E-5 
                          //    3.55E-5 
                          //----------------------------------------------------
  double  gen_swip_span;  //   (0.1 )
                          //----------------------------------------------------
  ULONG   gen_pulse_T;    //   
                          //----------------------------------------------------
  ULONG   gen_pulse_D;    //  
                          //----------------------------------------------------
  ULONG   gen_mod_K;    //    :
                          //  -   AM (0.1%)
                          //  -    (+-1.48939 )
                          //----------------------------------------------------
#define GEN_FM_MAX_DEV   195215.64318 //  
#define GEN_FM_STEP_DEV  1.48939      //  
#define GEN_FM_MAX_UINT  ( ( ULONG ) ( GEN_FM_MAX_DEV / GEN_FM_STEP_DEV ) )
                          //----------------------------------------------------
                          //     = +-195215.64318 
                          //  -     (0.1%)
                          //----------------------------------------------------
  USHORT    gen_ampl;       //   (4.088/4095 )
                          //----------------------------------------------------
                          //      
  UCHAR   gen_mod_zero1;  //  1
  UCHAR   gen_mod_zero2;  //  2
}                         //----------------------------------------------------
OSC, *POSC;
/*
================================================================================
                      PV65
================================================================================
*/
typedef struct __OSCCTRL
{
                          //----------------------------------------------------
                          //    :
  UCHAR   FPGA_ver;       //  -  FPGA
  UCHAR   firm_ver;       //  -  
  ULONG   serial;         //  -  
  UCHAR   pv_type;        //  -  
                          //----------------------------------------------------
#define TYPE_PV6501A 0x001C // PV6501A
#define TYPE_PV6502  0x0019 // PV6502
#define TYPE_PV6503  0x001F // PV6503
                          //----------------------------------------------------
                          //  TDS
  UCHAR   TDC_Rise;       //
  UCHAR   TDC_Fall;       //
  UCHAR   TDC_Rise2;      //
  UCHAR   TDC_Fall2;      //
                          //----------------------------------------------------
  USHORT  Vcc_Scope;      //  
                          //----------------------------------------------------
  USHORT  Points_Num;
                          //----------------------------------------------------
} OSCCTRL, *POSCCTRL;
/*
================================================================================
                             
================================================================================
*/
//==============================================================================
//                  pv_List   PV65XX
// : PV_SERIAL_LIST strs -      
// :
//    < 0 - 
//------------------------------------------------------------------------------
extern "C" int _DECLSPEC __stdcall pv_List( PV_SERIAL_LIST strs );
//==============================================================================

//==============================================================================
//             pv_Open     
//------------------------------------------------------------------------------
// :
//   - char* ser -     
// :
//   - DWORD* hnd -    
//    < 0 -  -    
//------------------------------------------------------------------------------
extern "C"  int _DECLSPEC __stdcall pv_Open( char* ser, DWORD* hnd );
//==============================================================================

//==============================================================================
//                 pv_Close    
//------------------------------------------------------------------------------
// :
//   - DWORD* h_pv -    
// :
//    < 0 - 
//------------------------------------------------------------------------------
extern "C"  int _DECLSPEC __stdcall pv_Close( DWORD * h_pv );
//==============================================================================

//==============================================================================
//                  pv_PutAllReg   
//------------------------------------------------------------------------------
// :
//   - DWORD* h_pv -    
//   - POSC osc -    OSC   
// :
//    < 0 -    
//------------------------------------------------------------------------------
extern "C"  int _DECLSPEC __stdcall
  pv_PutAllReg ( DWORD* h_pv , POSC osc );
//==============================================================================

//==============================================================================
//                pv_GetData     
//------------------------------------------------------------------------------
// :
//   - DWORD* h_pv -    
//   - PUCHAR A1, A2, D1, D2 -      
//    MAX_BUF_LENGTH
//    NULL -   
//   - POSC osc -    OSC    
//   - POSCCTRL oscctrl -    OSCCTRL - 
//   ,     
//------------------------------------------------------------------------------
// :
#define   PV_NO_DATA            0x0000 //  
#define   PV_PRESENT_A1         0x0001 //     A1
#define   PV_PRESENT_A2         0x0002 //     A2
#define   PV_PRESENT_D1         0x0004 //     D1
#define   PV_PRESENT_D2         0x0008 //     D2
#define   PV_PRESENT_CTRL       0x0010 //   CTRL
#define   PV_PRESENT_OSC        0x0020 //   OSC
#define   PV_UNCONFIG           0x0040 //  
#define   PV_CONFIGURED         0x0080 //     
#define   PV_PRESENT_ALL_DATA   0x000F //   : A1, A2, D1, D2
//------------------------------------------------------------------------------
extern "C"  int _DECLSPEC __stdcall
  pv_GetData ( DWORD* h_pv , PCHAR A1 , PCHAR A2 , PCHAR D1 , PCHAR D2 ,
               POSC osc , POSCCTRL oscctrl );
//==============================================================================
//  pv_CloseAll -     
//------------------------------------------------------------------------------
extern "C"  void _DECLSPEC __stdcall
   pv_CloseAll( void );
//==============================================================================
//                ""  OSC
// pv_PutOsc     -     
// pv_PutGen     -     
// pv_Config     -   ""  OSC
// pv_GetAllData -  
//------------------------------------------------------------------------------
//     att, zero, trig_lev, trig_gist,
#define CH_A1   0
#define CH_A2   1
#define CH_D1   2
#define CH_D2   3
//------------------------------------------------------------------------------
//       
#define  TRIG_P_MIN  0
#define  TRIG_P_MAX  1
//------------------------------------------------------------------------------
extern "C"  int _DECLSPEC __stdcall
   pv_PutOsc
   (
     DWORD*  h_pv,
     UCHAR   adc_mode,       //     
     USHORT  trig_mode,      //     
     UCHAR   trig_cond,      //      
     ULONG   smpl_per,       //    10ns
     USHORT  smpl_pre,       //  
     USHORT  smpl_post,      //    
     PUCHAR  att,            //  :         A1, A2
     PUSHORT zero,           //   :       A1, A2
     PUCHAR  trig_lev,       //   :   A1, A2
     PUCHAR  trig_gist,      //   : A1, A2
     UCHAR   trig_numb,      //      
     UCHAR   trig_slew,      //  
     PULONG  min_max_P,      //    
                             //   
     UCHAR   fmetr_in,       //   
     UCHAR   fps             //    ()
   );
//==============================================================================
// pv_PutGen
//------------------------------------------------------------------------------
//    
#define GEN_MAIN       0
#define GEN_MOD        1
#define GEN_SWEEP      2
//------------------------------------------------------------------------------
//      
#define GEN_PULSE_T    0
#define GEN_PULSE_D    1
//------------------------------------------------------------------------------
extern "C"  int _DECLSPEC __stdcall
   pv_PutGen
   (
     DWORD*  h_pv,
     bool    gen_enable,     //  
     UCHAR   gen_mode,       //  
     UCHAR   gen_mod_in,     //   
     bool    gen_mod_invert, //   
     UCHAR*  gen_shape,      //      
     double* gen_freq_span,  // 0 -  
                             // 1 - c     3.55E-5 
                             // 2 -   (0.1 )
     PULONG  gen_pulse,      //      
     ULONG   gen_mod_K,      //    AM    
     USHORT  gen_ampl,       //   (4.088/4095 )
     PUCHAR  gen_mod_zero    //  1, A2
   );

extern "C"  int _DECLSPEC __stdcall
   pv_Config ( DWORD*  h_pv );
//==============================================================================
// pv_GetAllData
//------------------------------------------------------------------------------
// Vb, Va -    , S -  
#define  TRIG_VB    0
#define  TRIG_VA    1
#define  TRIG_S     2
//------------------------------------------------------------------------------
//     
#define  SMPL_PRE   0
#define  SMPL_POST  1
//------------------------------------------------------------------------------
//  FPGA,  Firmware,  , Vpp
#define  FPGA_VER     0
#define  FRIM_VER     1
#define  PV_TYPE      2
#define  VPP_SCOPE_H  3
#define  VPP_SCOPE_L  4
//------------------------------------------------------------------------------
extern "C"  int _DECLSPEC __stdcall
   pv_GetAllData
   (
     DWORD*  h_pv,
     UCHAR*  A1,             // 
     UCHAR*  A2,             // 
     UCHAR*  D1,             // 
     UCHAR*  D2,             // 
     PUCHAR  flags,          //  
#define EQ_FLAG     0x01     //     
#define END_FLAG    0x02     //      
#define FREQ_VALID  0x04     //    
     PUSHORT  addr_eq,       //      
     PULONG   trig_fine_pos, //   
     UCHAR*   trig_Vb_Va_S,  //  Vb, Va -    
                             //  S -  
     PUSHORT  trig_P,        //    
     double*  trig_gen_phase,//     
     double*  freq,          //  

     USHORT*   smpl,         //     
     UCHAR*    ver_type_V    //  FPGA,  Firmware,  , Vpp
   );
#endif
